

#include "discoord.h"


/* **********************************************************
********************************************************** */
int DIS_emissions_trackjam_type::build(int *count, int *systemsize, int *beamsize, sfile *sbuffer) {

   *count += DIS_PDU_SIZE_BASE_EMISSIONS_TARGET;
   *systemsize += DIS_PDU_SIZE_BASE_EMISSIONS_TARGET_32;
   *beamsize += DIS_PDU_SIZE_BASE_EMISSIONS_TARGET_32;

   sbuffer->sput_ushort(site_id);
   sbuffer->sput_ushort(application_id);
   sbuffer->sput_ushort(entity_id);
   sbuffer->sput_uchar(emitter_id);
   sbuffer->sput_uchar(beam_id);

   return 1;
}


/* **********************************************************
********************************************************** */
void DIS_emissions_trackjam_type::extract(sfile *sbuffer) {
   
   site_id = sbuffer->scan_ushort();
   application_id = sbuffer->scan_ushort();
   entity_id = sbuffer->scan_ushort();
   emitter_id = sbuffer->scan_uchar();
   beam_id = sbuffer->scan_uchar();
}


/* **********************************************************
********************************************************** */
int DIS_emissions_beams_type::build(int *count, int *systemsize, sfile *sbuffer) {

   int ret = 1;
   DIS_emissions_trackjam_type *ptr;
   unsigned char *ctr, *dtr, *etr;
   int beamsize;
   int i;
   
   *count += DIS_PDU_SIZE_BASE_EMISSIONS_BEAM;
   *systemsize += DIS_PDU_SIZE_BASE_EMISSIONS_BEAM_32;

   ctr = sbuffer->ptr;
   
   sbuffer->sput_uchar(0); // beam_size;
   sbuffer->sput_uchar(beam_id);
   sbuffer->sput_ushort(beam_parameter_index);

   sbuffer->sput_float(frequency);
   sbuffer->sput_float(freq_range);
   sbuffer->sput_float(power);
   sbuffer->sput_float(prf);
   sbuffer->sput_float(pulse_width);
   sbuffer->sput_float(beam_azimuth);
   sbuffer->sput_float(beam_az_sweep);
   sbuffer->sput_float(beam_elevation);
   sbuffer->sput_float(beam_ele_sweep);
   sbuffer->sput_float(beam_sweep_sync);

   sbuffer->sput_uchar(beam_function);

   etr = sbuffer->ptr;
   
   sbuffer->sput_uchar(0); // target_count
   sbuffer->sput_uchar(high_density);
   sbuffer->sput_uchar(0);

   sbuffer->sput_uint(jamming_mode);

   for (i = 0, beamsize = DIS_PDU_SIZE_BASE_EMISSIONS_BEAM_32, ptr = (DIS_emissions_trackjam_type *)trackjamman.head; ptr && ret; ptr = (DIS_emissions_trackjam_type *)ptr->next)
      if (*count + DIS_PDU_SIZE_BASE_EMISSIONS_TARGET <= DIS_PDU_SIZE_MAX_PDU_SIZE) {
         ret &= ptr->build(count, systemsize, &beamsize, sbuffer);
         i++;
      }
      
      else
         ret = 0;
	 
   dtr = sbuffer->ptr;

   sbuffer->ptr = ctr;
   sbuffer->sput_uchar(beamsize);

   sbuffer->ptr = etr;
   sbuffer->sput_uchar(i);

   sbuffer->ptr = dtr;

   return ret;
}


/* **********************************************************
********************************************************** */
void DIS_emissions_beams_type::extract(sfile *sbuffer) {

   int count;
   DIS_emissions_trackjam_type *ptr;
   int i;   
   
   sbuffer->scan_uchar(); // beam_size;
   beam_id = sbuffer->scan_uchar();
   beam_parameter_index = sbuffer->scan_ushort();

   frequency = sbuffer->scan_float();
   freq_range = sbuffer->scan_float();
   power = sbuffer->scan_float();
   prf = sbuffer->scan_float();
   pulse_width = sbuffer->scan_float();
   beam_azimuth = sbuffer->scan_float();
   beam_az_sweep = sbuffer->scan_float();
   beam_elevation = sbuffer->scan_float();
   beam_ele_sweep = sbuffer->scan_float();
   beam_sweep_sync = sbuffer->scan_float();

   beam_function = sbuffer->scan_uchar();
   count = sbuffer->scan_uchar();
   high_density = sbuffer->scan_uchar();
   sbuffer->scan_uchar();
   
   jamming_mode = sbuffer->scan_uint();

   trackjamman.dest();
   
   for (i = 0; i < count; i++) {
      trackjamman.append(ptr = new DIS_emissions_trackjam_type, NULL);
      ptr->extract(sbuffer);
   }

}


/* **********************************************************
********************************************************** */
int DIS_emissions_systems_type::build(int *count, sfile *sbuffer) {

   int ret = 1;
   int i, systemsize;
   DIS_emissions_beams_type *ptr;
   unsigned char *ctr, *dtr;
   
   *count += DIS_PDU_SIZE_BASE_EMISSIONS_SYSTEM;
   
   ctr = sbuffer->ptr;
   
   sbuffer->sput_uchar(0); // size;
   sbuffer->sput_uchar(0); // beam_count;
   sbuffer->sput_uchar(0);
   sbuffer->sput_uchar(0);

   sbuffer->sput_ushort(emitter_name);
   sbuffer->sput_uchar(emitter_function);
   sbuffer->sput_uchar(emitter_id);

   sbuffer->sput_float(location[0]);
   sbuffer->sput_float(location[1]);
   sbuffer->sput_float(location[2]);

   for (i = 0, systemsize = DIS_PDU_SIZE_BASE_EMISSIONS_SYSTEM_32, ptr = (DIS_emissions_beams_type *)beamman.head; ptr && ret; ptr = (DIS_emissions_beams_type *)ptr->next)
      if (*count + DIS_PDU_SIZE_BASE_EMISSIONS_BEAM + DIS_PDU_SIZE_BASE_EMISSIONS_TARGET <= DIS_PDU_SIZE_MAX_PDU_SIZE) {
         ret &= ptr->build(count, &systemsize, sbuffer);
         i++;
      }
      
      else
         ret = 0;
      
   dtr = sbuffer->ptr;

   sbuffer->ptr = ctr;
   sbuffer->sput_uchar(systemsize);
   sbuffer->sput_uchar(i);

   sbuffer->ptr = dtr;

   return ret;
}


/* **********************************************************
********************************************************** */
void DIS_emissions_systems_type::extract(sfile *sbuffer) {

   int count;
   DIS_emissions_beams_type *ptr;
   int i;
   
   sbuffer->scan_uchar(); // size;
   count = sbuffer->scan_uchar(); // size;
   sbuffer->scan_uchar();
   sbuffer->scan_uchar();

   emitter_name = sbuffer->scan_ushort();
   emitter_function = sbuffer->scan_uchar();
   emitter_id = sbuffer->scan_uchar();

   location[0] = sbuffer->scan_float();
   location[1] = sbuffer->scan_float();
   location[2] = sbuffer->scan_float();

   beamman.dest();
   
   for (i = 0; i < count; i++) {
      beamman.append(ptr = new DIS_emissions_beams_type, NULL);
      ptr->extract(sbuffer);
   }

}


/* **********************************************************
********************************************************** */
DISPDU_emissions_type::DISPDU_emissions_type() {

   header.type = DIS_PDU_TYPE_EMISSIONS;
   header.family = DIS_PDU_FAMILY_EMISSION_REGENERATION;
   header.size = DIS_PDU_SIZE_BASE_EMISSIONS;
   emitting_site_id = dis_site_id;
   emitting_application_id = dis_application_id;
   event_site_id = dis_site_id;
   event_application_id = dis_application_id;
   state_update_indicator = DIS_EMISSION_STATE_UPDATE;
}


/* **********************************************************
********************************************************** */
int DISPDU_emissions_type::build(buffer_type *buffer) {

   sfile sbuffer;
   DIS_emissions_systems_type *ptr;
   int ret = 1;
   int count = header.size;
   int i;
   
   sbuffer.dest();

   header.build(&sbuffer);

   sbuffer.sput_ushort(emitting_site_id);
   sbuffer.sput_ushort(emitting_application_id);
   sbuffer.sput_ushort(emitting_entity_id);

   sbuffer.sput_ushort(event_site_id);
   sbuffer.sput_ushort(event_application_id);
   sbuffer.sput_ushort(event_entity_id);

   sbuffer.sput_uchar(state_update_indicator);
   
   sbuffer.sput_uchar(0);   // systemman.count;
   sbuffer.sput_uchar(0);
   sbuffer.sput_uchar(0);

   for (i = 0, ptr = (DIS_emissions_systems_type *)systemman.head; ptr && ret; ptr = (DIS_emissions_systems_type *)ptr->next)
      if (count + DIS_PDU_SIZE_BASE_EMISSIONS_SYSTEM + DIS_PDU_SIZE_BASE_EMISSIONS_BEAM + DIS_PDU_SIZE_BASE_EMISSIONS_TARGET <= DIS_PDU_SIZE_MAX_PDU_SIZE) {
         ret &= ptr->build(&count, &sbuffer);
         i++;
      }
      
      else
         ret = 0;
      
   sbuffer.sseek(8);
   sbuffer.sput_ushort(count);

   sbuffer.sseek(25);
   sbuffer.sput_uchar(i);

   buffer->buffercpy((char *)sbuffer.data, sbuffer.vsize);
   return ret;
}


/* **********************************************************
********************************************************** */
void DISPDU_emissions_type::extract(buffer_type *buffer) {

   sfile sbuffer;
   int count;
   int i;
   DIS_emissions_systems_type *ptr;
   
   sbuffer.scan_data(buffer);

   header.extract(&sbuffer);

   emitting_site_id = sbuffer.scan_ushort();
   emitting_application_id = sbuffer.scan_ushort();
   emitting_entity_id = sbuffer.scan_ushort();

   event_site_id = sbuffer.scan_ushort();
   event_application_id = sbuffer.scan_ushort();
   event_entity_id = sbuffer.scan_ushort();

   state_update_indicator = sbuffer.scan_uchar();
   count = sbuffer.scan_uchar();
   sbuffer.scan_uchar();
   sbuffer.scan_uchar();

   systemman.dest();
   
   for (i = 0; i < count; i++) {
      systemman.append(ptr = new DIS_emissions_systems_type, NULL);
      ptr->extract(&sbuffer);
   }

}


/* **********************************************************
********************************************************** */
void DISPDU_emissions_type::setup(int id) {

   systemman.dest();

   emitting_entity_id = id;
}
