

#include "global.h"
#include "vdis.h"


/* **************************************************************
************************************************************** */
void dis_platform::update_data(double *machine_time) {


// Note:  This platform will have to get updated when PDU's are
//        received... not here

}


/* **************************************************************
************************************************************** */
void dis_platform::update(dis_iface *isim, DISPDU_entity_state_type *es, double *machine_time) {

   vector3d world;
   vector3d worldypr, localypr;

   if (es->appearance_flag & DIS_PDU_APPEARANCE_DESTROYED) {
      platform_flag |= PLATFORM_FLAG_DEAD | PLATFORM_FLAG_POSTMORTIUM;
      perform(isim, PERFORM_PLATFORM_DEACTIVATE, NULL);
      return;
   }

   copyarray3(world, es->location);
   dis2local(world, plat_clone.init_pos, DIS_POSITION);
   copyarray3(plat_clone.pos, plat_clone.init_pos);

   copyarray3(world, es->velocity);
   dis2local(world, plat_clone.init_vel, DIS_VECTOR);
   copyarray3(plat_clone.vel, plat_clone.init_vel);

   copyarray3(world, es->dr_accel);
   dis2local(world, plat_clone.acc, DIS_VECTOR);

   copyarray3(worldypr, es->ypr);
   dis2localypr(worldypr, localypr);

   plat_clone.rpy[0] = localypr[2];
   plat_clone.rpy[1] = localypr[1];
   plat_clone.rpy[2] = localypr[0];

//adsf dypr accel

   plat_clone.dead_recogn_flag =
      dotproduct3(plat_clone.vel, plat_clone.vel) > CORRECT ||
      dotproduct3(plat_clone.acc, plat_clone.acc) > CORRECT ||
      dotproduct3(plat_clone.drpy, plat_clone.drpy) > CORRECT;

   platform_flag |= PLATFORM_FLAG_MOVEMENT;
   plat_clone.elapsed_time = 0;
   plat_clone.timer = plat_clone.machine_time = *machine_time;
}


/* **************************************************************
************************************************************** */
int dis_platform::init(dis_iface *isim, DISPDU_entity_state_type *es, symbol_correlation_type *str) {

   correlate *basis;

   if (es) {
      dis_id.site = es->site_id;
      dis_id.application = es->application_id;
      dis_id.entity = es->entity_id;
   }

   id = isim->id_counter;
   isim->id_counter++;
   
   // find type
   if (!isim->platman.head) {
      pprintf("No correlation table\n");
      return 0;
   }

   basis = ((correlate *)isim->platman.head)->find(str->sym_3D);

   if (!basis) {
      sprintf(perror_buffer, "Warning:Bad Correlation Code %d\n", str->sym_3D);
      pprintf(perror_buffer);
      return 0;
   }

   if (((str->sym_3D/1000) % 100) == 2 || ((str->sym_3D/100000000) % 100) == 2)
      platform_flag |= PLATFORM_FLAG_FLY;
   
   // create rendering object
   if (vis_data) {
      delete vis_data;
      vis_data = NULL;
   }

   vis_data = ((platform_data *(*)(basic_platform *))isim->data_loader)(this);

   if (!vis_data)
      return 0;
  
   if (!vis_data->init(isim, basis, str)) {
      delete vis_data;
      vis_data = NULL;
      return 0;
   }

   return 1;
}


/* **************************************************************
************************************************************** */
dis_mcp_platform::dis_mcp_platform() {

   next_update_time = update_time = -1.0;
}


/* **************************************************************
************************************************************** */
int dis_mcp_platform::init(dis_iface *isim, symbol_correlation_type *str) {

   DIS_entity_type_type type;
   int t[8];

   if (!dis_platform::init(isim, NULL, str))
      return 0;

   sscanf(str->sym_sim.string, "%d:%d:%d:%d:%d:%d:%d:%d", &t[0], &t[1], &t[2], &t[3], &t[4], &t[5], &t[6], &t[7]);
   type.kind = t[0];
   type.domain =  t[1];
   type.country = t[2];
   type.category = t[3];
   type.subcategory = t[4];
   type.specific = t[5];
   type.extra = t[6];

   es.setup(dis_id.entity,
      type.country == DIS_COUNTRY_USA ? DIS_FORCE_FRIENDLY : DIS_FORCE_OPPOSING,
      &type, DIS_DR_RVW);

   es.header.time = 0;

   es.velocity[0] = 0;
   es.velocity[1] = 0;
   es.velocity[2] = 0;

   es.location[0] = 0;
   es.location[1] = 0;
   es.location[2] = 0;

   es.ypr[0] = 0;
   es.ypr[1] = 0;
   es.ypr[2] = 0;

   es.dr_accel[0] = 0;
   es.dr_accel[1] = 0;
   es.dr_accel[2] = 0;

   es.dr_ypr_velocity[0] = 0;
   es.dr_ypr_velocity[1] = 0;
   es.dr_ypr_velocity[2] = 0;

   update_time = 4.0 + 0.01*(id % 100);  // 5 seconds is the default DIS update time

   return 1;
}


/* **************************************************************
************************************************************** */
void dis_mcp_platform::update_platform(plat_stat_type *pc) {

   int manditory_flag;
   vector3d local, world;
   vector3d localypr, worldypr;
   vector3d diff, ediff;
   vector3d world_vel, world_accel;
   double t;

   manditory_flag = (platform_flag & PLATFORM_FLAG_FORCED_UPDATE) || next_update_time < pc->machine_time;

   copyarray3(local, pc->pos);
   local2dis(local, world, DIS_POSITION);

   localypr[0] = pc->rpy[2];
   localypr[1] = pc->rpy[1];
   localypr[2] = pc->rpy[0];
   local2disypr(localypr, worldypr);

   subeqarray3(diff, es.location, world);
   subeqarray3(ediff, es.ypr, worldypr);
   
   if (manditory_flag || dotproduct3(diff, diff) > 1.0 || dotproduct3(ediff, ediff) > CORRECT) {

      plat_clone = *pc;
   
      copyarray3(local, plat_clone.vel);
      local2dis(local, world_vel, DIS_VECTOR);

      copyarray3(local, plat_clone.acc);
      local2dis(local, world_accel, DIS_VECTOR);
   
      es.dr_ypr_velocity[0] = (float)-plat_clone.drpy[2];
      es.dr_ypr_velocity[1] = (float)plat_clone.drpy[1];
      es.dr_ypr_velocity[2] = (float)plat_clone.drpy[0];

      copyarray3(es.location, world);
      copyarray3(es.velocity, (float)world_vel);
      copyarray3(es.dr_accel, (float)world_accel);

      copyarray3(es.ypr, (float)worldypr);
//asdf accel

      t = plat_clone.timer;
      es.header.time = distime(&t);

      es.build(&send_buffer);
      disiface.send_message(&send_buffer);

      next_update_time = plat_clone.machine_time + update_time;
   }
   
}
